# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .statement_type import StatementType
import pydantic
import typing_extensions
from ..core.serialization import FieldMetadata
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Statement(UniversalBaseModel):
    id: str
    statement: str
    type: typing.Optional[StatementType] = pydantic.Field(default=None)
    """
    The type/category of the statement.
    """

    is_formalized: typing.Optional[bool] = None
    formalization: typing.Optional[typing.List[typing.Optional[typing.Any]]] = None
    z_3_formalization: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="z3_formalization")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
