# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .pic_component_info_value import PicComponentInfoValue
from .pic_component_settings_value import PicComponentSettingsValue
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class PicComponent(UniversalBaseModel):
    component: str
    info: typing.Optional[typing.Dict[str, PicComponentInfoValue]] = None
    settings: typing.Optional[typing.Dict[str, PicComponentSettingsValue]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
