# This file was auto-generated by Fern from our API Definition.

import typing

StatementType = typing.Union[
    typing.Literal["COST_FUNCTION", "STRUCTURE_CONSTRAINT", "PARAMETER_CONSTRAINT", "QUERY_SUMMARIZER", "UNKNOWN"],
    typing.Any,
]
